
/*
Demo on file handling in SAS.
Note: you can place comments by surrounding text by slash and asterisk. Or, one
can start a comment with an asterisk (and ending with a semi-colon).

Remember: all instructions or statements in SAS must end in a semi-colon.
*/

*---SAS program files are just text files (or enhanced text). Only with execution
	does something happen.;
*---There are PROCedures and 'data' steps (special kinds of PROCs).;


title "Demonstration of SAS file and manipulation";

data a;
input x y code id;	
datalines;
100 1 0 1  
200 3 0 2
500 4 0 5
400 5 0 4
300 1 1 3
700 2 1 7
600 3 1 6 
800 7 1 8
;
run;
proc print data=a;run;

data a; set a;			*<--to work on a previously created file, specify the file ("a"),
							then "set" the file ("a") into the active file.; 
z = x+y;
run;

data b;
input x y;
datalines;
3 7 
4 9
7 9
;
data c; set a b;				*<---stack data (if variables are not present, they become missing.;
								*<--Use the set command with the list of stacked files;
proc print data=c;run;


data a2;
input x2 name $ code2 id;		*<--inputting character variables (need a $);
datalines;
50 bill 0 5  
20 joe 0 2
10 kim 0 1
40 sam 0 4
30 lynn 1 3
80 joe 1 8
60 john 1 6 
70 jen 1 7
90 gaby 2 9
;

*---One can merge files (side-by-side), by identifying a variable for pairing records;
*---Must sort by that identifier first;

proc sort data=a; by id; run;
proc sort data=a2; by id; run;

data a_; merge a a2; by id;

proc print data=a_; run;

data a_; set a_;
	name2 = propcase(name);			*<--"Proper Case function: Make sure the first 
										letter is a capital;
proc print data=a_;
	var ID name2 x2 x y;
run;

proc sort data=a_ out=a_sort;
	by name2;
run;
proc print data=a_sort;
	var ID name2 x2 x y;
run;


data a_stack; set a a2;
proc print data=a_stack; run;


data one;
x1=0; x2=100; y1=100; y2=5.0;
run;

proc print data=one; run;

data d;								*<--string numbers across records (use "@@");
input z @@;
datalines;
1 3 10 29 20
1 3 9 10
3
10 10 10 19 6 7 1 9 9 7 1 2 5 5 5 10 9
17 10 9
;

data ee;
input variety $ y @@ ;					*<--stringing numbers (in PAIRS) across records;
datalines;
Abc 30  Alissa 50
Temp 10 Mike 18  Joe	40 andy 0
Kylea 50
;
run;

proc print data=ee;run;
proc sort data=ee; by variety; run;
proc print data=ee;run;

data ee2; set ee; 
	if (substr(variety,1,1) eq 'A') then delete;	*<--Look at pieces or character variables;
proc print data=ee2;
run;

*****************************************************;

data ee3; set ee;
	variety = upcase(variety);
proc sort data=ee3;
	by variety;
proc print data=ee3;

data ee4; set ee3;  								*<--Demo: remove records when first letter
														of VARIETY is an "A";
	if (substr(variety,1,1) eq 'A') then delete;
													*<--Demo of IF - THEN statements;
proc print data=ee4; run;

*****************************************************;

proc print data=a;run;

data astack; set a;									*<--Option to do INTERNAL stacking;
response = x;										*<--(Create two records for each original);
treatment = 1;										*<--(Make X and Y the new RESPONSE variable);
output;
response = y;
treatment = 2;
output;
run;
proc print data=astack; var treatment response ; run;




/* There are other ways of reading in data */
/* NOTE: YOU MUST PUT IN THE PROPER PATH FOR YOUR COMPUTER */

data f ;
infile 'C:\Documents and Settings\madden.1\My Documents\My SAS Files\demo10\demo1.dat';
input x1 x2;
run;

proc print data=f(obs=20);						*<--Note: one can print subsets of total;
run;

*---IMPORT is a special PROC for pulling in special files (such as excel files).;
proc import 
  datafile='C:\Documents and Settings\madden.1\My Documents\My SAS Files\demo10\demo2.xls'
  out = demo2s replace;		
  sheet = sheet1;			*<--go to sheet1 of the excel file;
  getnames=no;				*<--option to NOT use the first record as variable names;

proc print data=demo2s;run;

data demo2s; set demo2s;
	temper = f1; rh = f2; wind=f3;
	drop f1 f2 f3;

proc print data=demo2s;
run;


*---Learn about LIB NAME. This is very subtle, but very useful in creating and 
	retrieving permanent SAS files from a disc.
	The 'name' given is arbitrary.;
*---Note: the LIBNAME statement does not give the file name, just the path to the folder;
*---Only needed once per program;

libname demo 'C:\Documents and Settings\madden.1\My Documents\My SAS Files\demo10';

proc print data=a_;run;

data demo.a_; set a_;		*<--this just created a perm. SAS file on the C drive called a_;
run;						*<--can only be opened in SAS;

proc print data=demo.a_;run;

*---EXPORT is special PROC for creating special files (including Excel);
proc export data=a_ 
  outfile='C:\Documents and Settings\madden.1\My Documents\My SAS Files\demo10\demoout_a_.xls'
  replace;
run;

proc print data=a;run;
*---Now write an ascii (text) file;
data a; set a;
	file 'C:\Documents and Settings\madden.1\My Documents\My SAS Files\demo10\demoout_ascii_a.dat';
	put x y;
run;

*---Every procedure has its own syntax, athough there are some common features across many
	PROCs. Many PROCs have a VAR (variable) statement to identify variables for analysis.;
*---Remember: upper and lower case for sas commands does not matter;


proc print data=a_; var x2 x; run;

title2 'use of proc means';
proc means data=a_ mean median min max stderr skew n;  
var x x2; 													*<--easy way to get means and other summaries;
run;														*<--Desired statistics are listed on the 
															procedure statement (after data=_).;
run;

proc means data=a_ noprint ; 
	var x x2; 								*<--Many PROCs have OUTPUT statements for storage;
output out=a_out mean(x)=mnx mean(x2)=mn_x2 ;
run;

proc print data=a_out;run;

*---in MEANS OUTPUT, syntax is: function(input_variable) = output_variable_name .... ;

data two;								*<--More complex data structure;
input trt $ rep subs y;
datalines;
a 1 1 100
a 1 2 200
a 2 1 399
a 2 2 300
a 2 3 401
a 2 4 2e2
a 3 1 3e4
a 3 2 400
b 1 1 59
b 1 2 66
b 2 1 90
b 2 2 40
b 3 1 49
b 3 2 80
;
proc print data=two;
run;

*---The CLASS statement is important in many PROCs;

proc means data=two n mean var median;		*<--getting separate results for each "group";
class trt;
var y;
output out=two_out mean(y)=ybar var(y)=yvar median(y)=ymed;
run;
proc print data=two_out;
run;

data two_out; set two_out; 				*<--cleaning up the output file, for later use;
	if (_type_ eq 0) then delete;
	drop _type_ _freq_;
proc print data=two_out;
run;

*****************************************************************************************;
proc print data=two;run;


proc means data=two n mean var median  ;		*<--getting separate results for each "group";
class trt rep;
var y;
output out=two_2out mean(y)=ybar var(y)=yvar median(y)=ymed;
run;
proc print data=two_2out;
run;

data two_2outtemp; set two_2out; 
if (_type_ ne 3) then delete;
proc print data=two_2outtemp;run;


data two_comb; merge two two_out; by trt;		*<--Put treatment means in every record of
													original file, next to raw numbers.;
												*<--A type of MERGING;
proc print data=two_comb; run;

proc sort data=two out=two;
	by trt rep;
run;

*---Another way of getting separate results for each "group", using BY statement.
	Data must be sorted first with BY statements (but not with CLASS statements).;
proc means data=two ;							
by trt rep;						*<--separate results for each trt-rep combination;
var y;
output out=two_out2 mean(y)=ybar;
run;



proc print data=two_out2;		*<--with this approach, the output file does NOT have an
									overall mean record;
run;

*---Get overall mean with the MIXED procedure;
/* response = intercept + fixed-effects + random-effects + residual */
/* response = intercept +                                  residual */

proc mixed data=two;
model y = / solution ;		*<--fit a model with an implicit 'intercept' (constant) and residual;
run;

proc means data=two mean var n ;
var y;
run;

*---Demo of an old-fashioned ANOVA (balanced only, no random effects);
*---One can use the Output Delivery System to customize output storage, and presentation
	of results. Example here: saving pieces of PROC output in files.;
*---Syntax: one must know the names that SAS gives to each piece of PROC output 
	(see Manuals). Then use ODS OUTPUT to give SAS file names to desired portions.;


*---IN GENERAL, NEVER USE PROC ANOVA;
proc print data=two_out2;run;

ods html path='C:\temp' (URL=NONE) body='sastest.html' ;
ods graphics on;
proc anova data=two_out2;
title2 'Demo of PROC ANOVA for class or workshop folks';
ods output means=means modelanova=modelanova;
class trt rep;
model ybar = rep trt ;
means trt ;
run;

ods graphics off;
ods html close;
run;

proc print data=means;run;
proc print data=modelanova;run;
data demo.meansdemo; set means;
data modelanova1; set modelanova;
	if (source eq 'rep') then delete;
run;
proc print data=modelanova1;run;

************************;


data miss;
input x y code id;

if (x = .) then delete;
if (y eq .) then delete;

z = y**2.5; 
a = log(x);  * log(.) means base e log, or ln(.) ; *log10(.) for base 10 log; 
a_round = round(a);
a_floor = floor(a);
x2 = x*2 + y/3 + x*y; 

if (x gt 400) then b = y;
	else b=0;

high = (x > 400);
big = (x > 400)*y;

datalines;
100 1 0 1  
200 3 0 2
500 4 0 5
400 5 0 4
300 1 1 3
700 2 1 7
600 3 1 6 
800 7 1 8
900 . 1 9
. 10 0 10
;
proc print data=miss;
title2 'data manipulation';
run;
;

proc print data=c;run;

proc sort data=c out=csort;
	by x;
run;

*---demo of SAS graphics (low-level version); /*Check out some simple concepts*/
proc print data=csort; run;

goptions reset=all;			*<--make sure the default options are chosen;
proc gplot data=csort; 
	plot y*x;
run;

title2 'look at graphics (classical) in SAS';
symbol1 v=dot h=2 l=1 w=2 c=blue  ;
proc gplot data=csort; 
	plot y*x;
run;

*---Now get much more sophisticated with controlling graphs;
*---One can define any number of axis properties, and identify the choices in a 
	plot statement;


axis1 order=(0 to 800 by 200) minor=(number=1) value=(font=swissb h=2)
	label = (f=swissb h=2 'Predictor (X)') width=3;
axis2 order=(0 to 10 by 2) minor=(number=4) value=(font=swissb h=2) 
	label = (angle=90 f=swissb h=2 'Response (Y)');

proc gplot data=csort; 
	plot y*x / haxis=axis1 vaxis=axis2 ;run;

proc gplot data=miss;
symbol1 v=dot h=2 l=1 w=2 c=blue i=none;
symbol2 v=none color=red l=1 h=3 i=join w=2;
	plot (y a)*x / overlay;
	plot (y a)*x / overlay haxis=axis1 vaxis=axis2;
run;

proc sort data=miss out=miss_sort;
	by x a y;
run;
proc gplot data=miss_sort;
	plot (y a)*x / overlay haxis=axis1 vaxis=axis2;
	
run;

*************************************************************************;

title2 't-tests in SAS';
proc print data=d;run;

*---Use SAS to perform a one-sample t-test;
proc ttest data=d  ; /* H0: mu =0 */
	var z;
run;
proc ttest data=d  H0=10 ; /* H0: mu =10 */
	var z;
run;

proc print data=two;run;
*---Two-sample t-test;
proc ttest data=two ;
	class trt;
	var y;
run;

*---UNIVARIATE is a very useful PROC for summarizing variables individually, especially
	for a large number of observations.;

title2 'univariate summaries';
proc univariate data=d;
var z;
run;

data simulate; 
	call streaminit(12345);				*<--lock in starting value for random numb. generator;

	do i = 1 to 10000;
		y = rand('POISSON',2);
		z = rand('GAMMA',2);
		output;
	end;

	ly = log(y);
run;
proc means data=simulate mean median min max var skew kurt n;
var y z;
run;

proc print data=simulate(obs=20);
var y z;
run;

goptions reset=all;
proc univariate data=simulate;
var z ;
histogram z / normal;
qqplot z / normal(mu=est sigma=est);
run;

proc univariate data=simulate;
var z;
histogram z / turnvlabels cfill=red cframe=white
	cframeside=white endpoints= 0 to 14 by 1 cbarline=black
	font=swissb height=3;
run;


proc print data=a;run;

ods html /*style=astronomy*/ path='C:\temp' (URL=NONE) body='sastest.html' ;
ods graphics on;
proc reg data=a;
model x = y;
run;
ods graphics off;
ods html close;
run;
