*---Demonstrate some use of IF-THEN statements, and other manipulations;
*--- demo_file manipulation3.sas ---;

title 'Demo of file manipuation, IF-THEN, Stacking, LABELS, etc. (file: ...3)';
data x;
input z y;

	if (z < 3) then trt = 1;		*<--Convert continuous variables into categories;
	else if (z < 6) then trt = 2;
	else trt = 3;
datalines;
9.8	19.3
2.3	6.3
2.2	6.9
0.4	5.3
9.2	21.3
1.0	4.9
5.4	12.4
5.3	13.1
8.2	17.5
5.6	13.8
1.2	8.8
9.5	27.3
8.1	17.5
6.6	14.2
0.3	4.6
0.4	2.2
3.6	7.5
5.9	17.6
7.2	18.4
6.6	15.6
3.9	8.6
3.5	7.6
5.2	11.0
5.1	13.7
6.5	15.6
;

title2 'Data file x';
proc print data=x;
run;

data x2; set x;
	if (z > 8);		*<--Looks like something is missing (no THEN part);
run;				*<--Actually, this means "if (z > 8) then KEEP, otherwise delete";

title2 'File x, with small z deleted (x2)';
proc print data=x2;run;

*---Demonstration of labeling variables for later output. Variable names are replaced
	by these labels, sometimes automatically (some procedures), and sometimes only
	with the right option (i.e., "label" in PRINT procedure).;

data x; set x;
	label y = 'Plant growth rate' trt = 'Plant Treatment' z = 'Measure of conditions';

proc print data=x label ;				*<--Need LABEL option for PRINT PROC;
	title2 'File x, with Labels for variables (requires LABEL option)';
	var z trt  y;
run;

proc means data=x;
class trt;
var y;
run;

*---Use FORMAT procedure to give more descriptive data labels (NOT variable labels)
	for the observations in a variable. This goes into effect in the following procedures
	_if_ FORMAT statement is given.;

proc format;										*<--Note: no data=____ part;
	value trtcode 1='Low disease' 2='Medium disease' 
		3='High disease';							*<--"trtcode" is an arbitrary name;
run;												*<--will reference selectively, below;


/* Example use of observation FORMAT for output */
proc print data=x label;
	title2 'File x, with Labels for variable (requires LABEL option)';
	title3 'and format labels (trtcode.) for observations in TRT variable';

	format trt trtcode.;	*<--Putting "trtcode." after variable invokes FORMAT PROC;
							*<--SAS can tell to use this format for TRT;
	var z trt  y;
run;

proc means data=x;			*<--Now put format in MEANS procedure output;
title2 'Summary stats for variable y, by TRT';
title3 'using observation format labels (trtcode.)';
format trt trtcode.;				*<--Without this otpional statement, this 'format' is not used;
class trt;
var y;
run;

title2 'Find out what is in file x using CONTENTS procedure';
proc contents data=x;run;			*<--very useful for complex files;

title2 'what data files do I have???';
proc datasets;run;


/**** NOW DEMONSTRATE STACKING OF MULTIVARIATE OBSERVATIONS ***/

data mult;
input rep x y1 - y10;
datalines;
1 1.5  1 10 5 5 6 10 9 1 20 19
2 3.3  2 9 5 7  6 15 7 3 21 20
3 2.5  1 12 4 5 5 11 9 3 14 16
4 4.5  4  15 6 7 7 16 10 4 14 15
5 1.9  1 9 4 4 4 14 8 2 19 16
;
run;


ods html;
ods graphics on;

title2 'Working with multivariate data';
title3 'Printing all the data';
proc print data=mult;
run;

/* 
Sometimes one wants to work on a portion of a data set (e.g., analyze only observations
if treatment = 1, etc.). One can create a new file 
		data NEW; set OLD; if (treatment = 1); run;
and then work with this file. However, there is a faster way, that does not require
creating a separate file (just process a portion of a file). Use the WHERE= option.
*/

title3 'Seeing a subset of data (x < 4)';
proc print data=mult(where=(x < 4));	*<--File MULT still has all observations, but
											only a subset are handled in this PRINT run;
run;

title3 'Printing out only the first 2 observations';
proc print data=mult(obs=2);
run;


title3 'Looking at correlations (Pearson product moment) of variables';
proc corr data=mult plots=matrix;
	var x y1 - y10;
run;

ods graphics off;
ods html close;
run;

/*
Previously, I demonstrated how to take different variables in a file, and stack them
in a new file. This is different from stacking two different files together.
The concept is to put 'multivariate' data in 'univariate' form. This may be
necessary for certain analyses (certain procedures expect to get the data in 
certain format styles. For instance, y1, y2, etc., may actually represent the
responses for treatments 1, 2, etc., with each record (line) corresponding to a 
different replicate.

Below, I first pretend there are only two variables to stack (y1 and y2). The values
for each observation (record) are stacked to make _two_ records, with RESPONSE 
representing y1 and y2. To indicate the treatment, I create the TREAT variable.
*/

title3 'Stacking y1 and y2 (into response), and flagging these with treat variable';
data mult2; set mult;
	response = y1;
	treat =1;
	output;					*<--always remember the output statement with each step;

	response = y2;
	treat = 2;
	output;
	drop y1 - y10 x;		*<--this is optional to get rid of cluttering of file;
run;
proc print data=mult2;
run;

*---I could now use PROC MEANS (etc.) to analyze response by treat.;

/*
If there were MANY variables (colums), maybe 50, 100, etc., the above would be very
tedious (but possible), with three lines of code for each variable. Fortunately,
there is a much more efficient method. This is based on DO loops.
In a DO loop, a sequence of one or more operations is performed multiple times.
Usually, some conditions changes with each journey through the loop.

In the following, I also introduce the idea of an ARRAY in a data step.
Arrays in SAS do not represent different observations, but different variables.
This is an efficient way of working with many variables in some organized manner.

Important: Do loops start with "do....;" and end with "end;". For this application,
there needs to be an "output;" within the loop (to write to a new file).

*/

title3 'Stacking y1-y10 into variable "response", with coding in "treat"';
data multstack; set mult;
	array ya{10} y1 - y10;				*---Define an array called ya with 10 elements;
										*---The elements correspond to y1 (1), y2 (2), etc.;
	do treat = 1 to 10;					*---Any name can be give for "treat";
		response = ya{treat};			*---Any name can be given for "response" or ya{.} ;
		output;
	end;
	drop y1 - y10 x;
run;									*---There can be several Do loops in data step;

proc print data=multstack;
run;

proc means data=multstack;
class treat;
var response;
run;


title3 'More complex stacking, with two factors (a and b)';
data multstack2; set mult;
	array ya{5} y1 - y5;  
	array yb{5} y6 - y10;

	do a = 1 to 5;
		b = 1;
		response = ya{a};
		output;
	end;

	do a = 1 to 5;
		b = 2;
		response = yb{a};
		output;
	end;
	drop y1 - y10;
run;

proc print data=multstack2;
run;


libname demo 'C:\Documents and Settings\madden.1\My Documents\My SAS Files\demo10';


data demo.multstack2; set multstack; run;	*<--Make a permanent copy of this new file.;




/*
When there are two or more factors, obtaining nice tables of means, medians, etc.,
can be challenging. The output from many procedures will give a long list of
factor-level combinations, and one must use brute-force to put these in 
easy-to-read formats. Consider the MEANS procdure...
*/
proc means data=multstack2;
class a b;
var response;
run;

proc mixed data=multstack2;			*<--will tell you about use of this PROC later!!!;
class a b;
model response = a b a*b;
lsmeans a b a*b;
run;

/* 
Sometimes, one wants to visualize the results directly in the output window, before
performing statistical analysis. The TABULATE procedure is very useful for this
purpose. The syntax for this procedure is different from most SAS procedures
(the PROC is based on a separate program developed by the U.S. Dept. of Labor). 
I like it for getting 2- and 3-way tables of summary statistics, together with 
formal analyses (using MIXED, etc.).

One can have three levels of synthesis (three factors).
The syntax is:
	table page, column, row;
With two factors, it is:
	table column, row;
With one factor, it is:
	table row;

In the simplist form, one obtains COUNTS of observations in each combination
of factor levels. To get means (etc.,) one must specify the desired statistic _and_
the response variable.
With two factors, syntax is:
	table column, row*mean*response;

*/

title3 'Use of TABULATE for summary tables (two factors)';
proc tabulate data=multstack2 /*format=8.2*/ ;
	class a b;								*<--Give the categories;
	var response x;							*<--Give the variables that might be summarized;
	table a, b;								*<--A very simple table (just giving counts);
	table a, b*mean*response;
	table a, b*median*response;
	table a, b*stddev*response;
	table a, b*mean*x;						*<--one can have any number of tables.;
	table b, a*mean*response;
run;

/*
One can get MARGINAL statistics also. Use:
	table (column all), (row all)*mean*response;
That is, put the row or column variable in parentheses, with the word ALL.
*/

proc tabulate data=multstack2 format=8.2;
	class a b;								*<--Give the categories;
	var response x;							*<--Give the variables that might be summarized;
											*<--A very simple table (just giving counts);
	table a, b*mean*response;
	table (a all),(b all)*mean*response;
	
	table (a all), (b all)*median*response;

	table (a all), (b all)*median*(response x);

	table (b all), (a all)*mean*response;
run;


*---Reminder: one can give labels to variables;
*---One can also give value labels to observations (substituting 
	the value label for the actual value, if a FORMAT is given;

data multstack22; set multstack2;
	label a = 'Student name' b = 'Incubation Temp.';
run;

proc format;					*<--the names are not replacing values, just associated with them.;
	value	first 1='Dan' 2='Alissa' 3='Kylea' 4='Cunyu' 5='Lee' ;
	value	second 1='hot' 2='cold';	
run;

title3 'Giving variable labels and value labels in stacked file';
proc print data=multstack22 label;
format a first. b second.;
run;

proc tabulate data=multstack22 format=8.2 ;
	class a b;
	var response x;
	format a first. b second. ;
	
	table (a all),(b all)*mean*response;
	
	table (a all), (b all)*median*(response x);
run;


*******************************************************************************;

run;


